"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sign = exports.signId = exports.ledgerSign = void 0;
const fs_1 = __importDefault(require("fs"));
const hw_transport_node_hid_1 = __importDefault(require("@ledgerhq/hw-transport-node-hid"));
const hw_app_avalanche_1 = __importDefault(require("@avalabs/hw-app-avalanche"));
const ethereumjs_util_1 = require("ethereumjs-util");
const utils_1 = require("./utils");
const output_1 = require("../output");
function ledgerSign(tx, derivationPath, blind = true) {
    var _a, _b, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        const message = blind ? tx.signatureRequests[0].message : tx.unsignedTransactionBuffer;
        const messageBuffer = Buffer.from(message, 'hex');
        const transport = yield hw_transport_node_hid_1.default.open(undefined);
        const avalanche = new hw_app_avalanche_1.default(transport);
        const { accountPath, signPath } = (0, utils_1.expandDerivationPath)(derivationPath);
        let pubk;
        let addr;
        let signature;
        if (blind) {
            const resp = yield avalanche.signHash(accountPath, [signPath], messageBuffer);
            if (resp.errorMessage != 'No errors') {
                throw new Error(`Can not sign message on ledger: ${resp.errorMessage}, code ${resp.returnCode}`);
            }
            const sign = (_b = (_a = resp.signatures) === null || _a === void 0 ? void 0 : _a.get(signPath)) === null || _b === void 0 ? void 0 : _b.toString('hex');
            if (!sign) {
                throw new Error("No signature returned");
            }
            signature = sign;
            pubk = (0, utils_1.recoverTransactionPublicKey)(messageBuffer, (0, utils_1.prefix0x)(signature));
            addr = (0, utils_1.recoverTransactionSigner)(messageBuffer, (0, utils_1.prefix0x)(signature));
        }
        else {
            const resp = yield avalanche.sign(accountPath, [signPath], messageBuffer);
            if (resp.errorMessage != 'No errors') {
                throw new Error(`Can not sign message on ledger: ${resp.errorMessage}, code ${resp.returnCode}`);
            }
            const sign = (_d = (_c = resp.signatures) === null || _c === void 0 ? void 0 : _c.get(signPath)) === null || _d === void 0 ? void 0 : _d.toString('hex');
            if (!sign) {
                throw new Error("No signature returned");
            }
            signature = sign;
            pubk = (0, utils_1.recoverTransactionPublicKey)((0, ethereumjs_util_1.sha256)(messageBuffer), (0, utils_1.prefix0x)(signature));
            addr = (0, utils_1.recoverTransactionSigner)((0, ethereumjs_util_1.sha256)(messageBuffer), (0, utils_1.prefix0x)(signature));
        }
        return {
            signature: signature,
            address: addr,
            publicKey: (0, utils_1.standardizePublicKey)(pubk.toString('hex'))
        };
    });
}
exports.ledgerSign = ledgerSign;
function signId(id, derivationPath, blind = true) {
    return __awaiter(this, void 0, void 0, function* () {
        return sign(`${id}.unsignedTx.json`, derivationPath, blind);
    });
}
exports.signId = signId;
function sign(file, derivationPath, blind = true) {
    return __awaiter(this, void 0, void 0, function* () {
        (0, output_1.logInfo)(`Please sign the transaction on your ledger device...`);
        const json = fs_1.default.readFileSync(file, 'utf8');
        const tx = JSON.parse(json);
        if (tx && tx.signatureRequests && tx.signatureRequests.length > 0) {
            const { signature } = yield ledgerSign(tx, derivationPath, blind);
            tx.signature = signature;
            let outFile = file.replace('unsignedTx.json', 'signedTx.json');
            if (outFile === file) {
                outFile = file + '.signed';
            }
            fs_1.default.writeFileSync(outFile, JSON.stringify(tx, null, 2));
        }
        else {
            throw Error("Invalid transaction file");
        }
    });
}
exports.sign = sign;
